/*
 *  Copyright (C) 2008 by National Chip Implementation Center
 *  Author: ycchang <yaochung@cic.org.tw>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include "hardware.h"

//#define	SWI	1
#undef SWI

unsigned long tick = 0;
 /* this function generates a s/w interrupt */
 
 void gen_swi (void)
 {
 	__asm__ __volatile__ (
 		"mfsr	$r0, $INT_PEND\n\t"
 		"li		$r1, #0x00010000\n\t"
 		"or		$r0, $r0, $r1\n\t"	
 		"mtsr	$r0, $INT_PEND\n\t"
 		:::"$r0", "$r1"					// $r0 and $r1 are clobbered registers
 	);
 }
 
 /*
  *  we disable the s/w, since there's no way to clear pendding interrupts 
  *  in IVIC mode
  */
  
 void clear_swi (void)
 {
 		__asm__ __volatile__ (
 		"mfsr	$r0, $INT_PEND\n\t"
 		"li		$r1, #0x00000000\n\t"
 		"and		$r0, $r0, $r1\n\t"	
 		"mtsr	$r0, $INT_PEND\n\t"
 		:::"$r0", "$r1"					// $r0 and $r1 are clobbered registers
 	);
 }
 
 void enale_global_interrupt (void)
 {
 	__asm__	__volatile__ (
 		"setgie.e\n\t"
 	);	
 }
 
 
 void intc_reset (void)
 {
 	/* Interrupt controller disabled all interrupts */
 	outw (INTC_BASE + IRQER_OFFSET, INT_IRQER_INIT);
 	/* clear all interrupt status 					*/
 	outw (INTC_BASE + IRQICR_OFFSET, INT_CLEAR_VALUE);
 }
 
 
 void intc_init (void)
 {
 	outw (INTC_BASE + IRQTLR_OFFSET, 0); 			// falling edge 
 	outw (INTC_BASE + IRQTMR_OFFSET, 1 << 0); 		// timer 1
 	outw (INTC_BASE + IRQER_OFFSET, 1 << 0);
 }
 
 void HW1_IRQ_Dispatcher (void)
 {
 	unsigned int int_status;
 	
 	#ifdef SWI	
 	sio_printf ("software interrupt occurred\n");
 	clear_swi ();
 	#else	
 	int_status = inw (INTC_BASE + IRQSR_OFFSET);
 	if (int_status & 0x1) {		// timer 0
 		//sio_printf ("IRQ 0\n");
 		outw (INTC_BASE + IRQICR_OFFSET, 0x1);
 		sio_printf ("\rtick is %d", tick);
 		tick++;
 	}
 	#endif
 }
