/*
 *  Copyright (C) 2008 by National Chip Implementation Center
 *  Author: ycchang <yaochung@cic.org.tw>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#define	IVB_offEVIC		13
#define	IVB_offESZ		14
#define	IVB_offIVBASE	16
#define	IVB_mskEVIC		(0x1UL << IVB_offEVIC)
#define	IVB_mskESZ		(0x3UL << IVB_offESZ)
#define	IVB_mskIVBASE	(0xFFFFUL << IVB_offIVBASE)

#define	INT_MASK_INIT	0x00001003
#define	ALLINT_MASK_N	0xfffeffc0


#define	vector_base 0x01E00000

	.extern main
	.extern __bss_start__
	.extern __bss_end__
	.extern HW1_IRQ_Dispatcher



	.equ	GIE_MASK_N,	0xfffffffe	! Global Interruption Enable
	.equ	GIE_MASK,	0x00000001	! Global Interrupt Enable Mask
	.equ	GIE_DIS,	0x00000000	! Global Interrupt Disable
	.equ	GIE_EN,		0x00000001	! Global Interruption Enable

	.equ	INTL_MASK_N,0xfffffff9	! Interrupt level mask
	.equ	INTL_LV0,	0x00000000	! Interruption level 0
	.equ	INTL_LV1,	0x00000002	! Interruption level 1

	.text
.LCvectors:
	.align	4	/* offset 0: Reset and NMI     */
	la	$p0, HandleReset
	jr	$p0
	
	.align	4	/* offset 1: TLB fill	       */
	la	$p0, HandleUndef
	jr	$p0

	.align	4	/* offset 2: PTE not present   */
	la	$p0, HandleUndef
	jr	$p0

	.align	4	/* offset 3: TLB misc	       */
	la	$p0, HandleUndef
	jr	$p0

	.align	4	/* offset 4: TLB VLPT miss     */
	la	$p0, HandleUndef
	jr	$p0
	
	.align	4	/* offset 5: Machine Error     */
	la	$p0, HandleUndef
	jr	$p0

	.align	4	/* offset 6: Debug related     */
	jr	$p0

	.align	4	/* offset 7: General exception */
	la	$p0, HandleUndef
	jr	$p0
	
	.align	4	/* offset 8: Syscall	       */
	la	$p0, HandleUndef
	jr	$p0
	
	.align	4	/* offset 9: HW 0	       */
	la	$p0, HandleUndef
	jr	$p0

	.align	4	/* offset 10: HW 1	       */
	la	$p0, HWHandle
	jr	$p0

	.align	4	/* offset 11: HW 2	       */
	la	$p0, HandleUndef
	jr	$p0

	.align	4	/* offset 12: HW 3	       */
	la	$p0, HandleUndef
	jr	$p0

	.align	4	/* offset 13: HW 4	       */
	la	$p0, HandleUndef
	jr	$p0

	.align	4	/* offset 14: HW 5	       */
	la	$p0, HandleUndef
	jr	$p0

	.align	4	/* offset 15: SW 0	       */
	la	$p0, HWHandle
	jr	$p0
	
HandleUndef:
1:	b	1b

HWHandle:
	pushm	$r0, $r30
	mfusr	$t0, $d0.hi
	mfusr	$t1, $d0.lo
	mfusr	$t2, $d1.hi
	mfusr	$t3, $d1.lo
	pushm	$t0, $t3
	
	! Call function to ISR dispatcher

	addi	$sp, $sp, #-24
	bal		HW1_IRQ_Dispatcher
	addi	$sp, $sp, #24
	popm	$t0, $t3
	mtusr	$t0, $d0.hi
	mtusr	$t1, $d0.lo
	mtusr	$t2, $d1.hi
	mtusr	$t3, $d1.lo
	popm	$r0, $r30
	
	iret


HandleReset:
	.global _start
_start:
	la	$sp, _stack					/* setup stack */
	la	$r0, __bss_start__			/* clear bss   */
	la	$r1, __bss_end__
	move	$r3, 0
	move	$r4, 0
	move	$r5, 0
	move	$r6, 0
1:
	smw.bim	$r3, [$r0], $r6
	sub		$p0, $r0, $r1
	bltz	$p0, 1b
	
	! disable data cache

	bal	processor_int_init
	bal	trap_init

	move	$r0, #0
	move	$r1, #0
	move	$fp, 0
	bal	main
1:
	b	1b
	
	/* 
	 * remember that we set 16 bytes for all 16 vectors
	 * for each entry when enable HSI	
	 *
	 */
trap_init:
	push	$lp
	! Set ESZ to 16 bytes	
	/* read interrupt vector base register */
	
	mfsr	$r4, $IVB	
	li		$r5, #(~IVB_mskESZ)
	and		$r4, $r4, $r5
	li		$r5, #0x4000			/* 16 bytes	*/
	or		$r4, $r4, $r5
	li		$r0, #vector_base
 	or		$r4, $r4, $r0
	mtsr	$r4, $IVB
	
	isb
	
	la		$p0, .LCvectors			/* set up vectors */
	movi	$r1, #16			
					
vector_loop:
	lmw.bim	$r2, [$p0], $r5
	smw.bim	$r2, [$r0], $r5
	addi	$r1, $r1, #-1
	bnez	$r1, vector_loop
	
	bal	invalid_cache_all

	pop	$lp
	ret

invalid_cache_all:
	! currenly, do nothing
	ret

processor_int_init:
	
	mfsr	$p0, $PSW
	li		$p1, #(GIE_MASK_N & INTL_MASK_N)
	and		$p0, $p0, $p1
	li		$p1, #(GIE_DIS | INTL_LV0)
	or		$p0, $p0, $p1
	mtsr	$p0, $PSW

	isb
	
	mfsr	$p0, $INT_MASK
	li		$p1, #ALLINT_MASK_N
	and		$p0, $p0, $p1
	li		$p1, #INT_MASK_INIT
	or		$p0, $p0, $p1
	mtsr	$p0, $INT_MASK

	ret
